'use strict';

const Promise = require('bluebird');
const maybePromised = require('../../temputils.js').maybePromised;

const execUnit = {
  up: function (context, driver, file) {
    let start;
    let end;
    const _file = file.get();

    if (_file.meta.noTransaction) {
      start = Promise.resolve;
      end = Promise.resolve;
    } else {
      start = context.driver.startMigration.bind(context);
      end = context.driver.endMigration.bind(context);
    }

    return start()
      .then(() => {
        return maybePromised(file, _file.migrate, [context.driver]);
      })
      .then(() => {
        return Promise.promisify(context.writeMigrationRecord.bind(context))(
          file
        );
      })
      .then(() => end());
  },

  down: function (context, driver, file) {
    return driver
      .startMigration()
      .then(() => {
        const _file = file.get();

        return maybePromised(file, _file.migrate, [context.driver]);
      })
      .then(() => {
        return Promise.promisify(context.deleteMigrationRecord.bind(context))(
          file
        );
      })
      .then(context.driver.endMigration.bind(context.driver));
  }
};

module.exports = execUnit;
